/*      -------------------------------------------------------------------
	xldlas -- A Stastics Package

	Copyright (C) 1996 Thor Sigvaldason

	Includes all the routines reached from the File menu.
	
        -------------------------------------------------------------------*/

#include "xldlas.h"

extern void quit_xldlas(FL_OBJECT *ob, long user_data);
extern void about_xldlas(FL_OBJECT *ob, long user_data);
extern void simple_line_output(char which_routine[XLDLASMAX_INPUT], char the_output[XLDLASMAX_INPUT]);
extern void say_status(char the_status[XLDLASMAX_INPUT]);
extern void inhibit_input();
extern void reenable_input();
extern void drop_variable(int to_drop, int shouldi);
extern int check_variable_name(char proposed[XLDLASMAX_INPUT]);



void drain_input(FILE *flptr)
{
	char drain;
	while(!feof(flptr))
	{
		fscanf(flptr,"%c",&drain);
	}
}

char *remove_blanks (char *inputstring)
{
	char working[XLDLASMAX_INPUT];
	int count;
	strcpy(working,inputstring);
	for(count = 0; count < strlen(working); count ++)
	{
		if(working[count] == ' ') working[count] = '_';
	}
	strcpy(inputstring,working);
	return(inputstring);
}

char *add_blanks (char *inputstring)
{
	char working[XLDLASMAX_INPUT];
	int count;
	strcpy(working,inputstring);
	for(count = 0; count < strlen(working); count++)
	{
		if(working[count] == '_') working[count] = ' ';
	}
	strcpy(inputstring,working);
	return(inputstring);
}


void load_a_variable(FILE *lfptr)
{
	char scratch[XLDLASMAX_INPUT];
	float nowhere;
	int count, place;
	if(numb_variables == MAX_VARS)
	{
		fl_show_alert("Dataset Contains Too Many Variables",
				"It was probably built with a bigger version of xldlas",
				"Please adjust MAX_VARS in xldlas.h and recompile xldlas",1);
		drain_input(lfptr);
		return;
	}
	fvector[numb_variables] = (float *) malloc (MAX_OBS * sizeof(float));
	if (!fvector[numb_variables])
	{
		fl_show_alert("Cannot Load all data in Dataset",
				"Running low on Memory",
				"",1);
		drain_input(lfptr);
		return;
	}
	fscanf(lfptr,"%s\n",scratch);
	strncpy(data_matrix[numb_variables].name, scratch,NAME_LENGTH);
	fscanf(lfptr,"%s\n",scratch);
	strncpy(data_matrix[numb_variables].description, scratch,DESC_LENGTH);
	fscanf(lfptr,"%i\n",&data_matrix[numb_variables].obs);
	if(data_matrix[numb_variables].obs > MAX_OBS)
	{
		strcpy(scratch,data_matrix[numb_variables].name);
		fl_show_alert("The Following Variable Contains Too Many Observations", scratch,"(I'm truncating it)",TRUE); 
	}
	data_matrix[numb_variables].fmem_start=fvector[numb_variables];
	place = 0;
	for(count = 0; count < data_matrix[numb_variables].obs; count ++)
	{
		if(place >= MAX_OBS)
		{
			fscanf(lfptr,"%f\n", &nowhere);
		}
		else
		{
			fscanf(lfptr,"%f\n", &*(fvector[numb_variables] + place));
			place++;
		}
	}
	if(data_matrix[numb_variables].obs > MAX_OBS) data_matrix[numb_variables].obs = MAX_OBS;
	numb_variables++;
}



void load_dataset(FL_OBJECT *ob, long user_data)
{
	FILE *lfptr;
	char string_one[XLDLASMAX_INPUT];
	int count;
	const char *filename;
	inhibit_input();
	say_status("Loading Dataset");
	fl_use_fselector(0);
	fl_set_fselector_title("Load a Dataset");
	filename = fl_show_fselector("Dataset to Load","","*.lda","");
	if(filename == NULL)
	{
		say_status("Ready");
		reenable_input();
		return;
	}
	lfptr = fopen(filename,"r");
	if(!lfptr)
	{
		fl_show_alert("Could not open requested file for reading","","",1);
		reenable_input();
		say_status("Ready");
		return;
	}
	for(count = numb_variables - 1; count >= 0; count--)
	{
		drop_variable(count,TRUE);
	}
	fscanf(lfptr,"%s\n",data_matrix_description);
	fscanf(lfptr,"%f\n",&missing_value);
	while(!feof(lfptr))
	{
		load_a_variable(lfptr);
	}
	fclose(lfptr);
	strcpy(data_matrix_description,add_blanks(data_matrix_description));
	for(count = 0; count <= numb_variables; count++)
	{
		strcpy(data_matrix[count].description,add_blanks(data_matrix[count].description));
		strcpy(data_matrix[count].name,add_blanks(data_matrix[count].name));
	}
	data_exists = TRUE;
	oktoquit = TRUE;
	sprintf(string_one,"Loaded %d variable(s) into memory from %s", numb_variables, filename);
	simple_line_output("load",string_one);
	reenable_input();
	say_status("Ready");
}




void save_dataset(FL_OBJECT *ob, long user_data)
{
	FILE *fptr;
	char string_one[XLDLASMAX_INPUT];
	int count,countt;
	const char *filename;
	inhibit_input();
	say_status("Saving Dataset");
	fl_use_fselector(1);
	fl_set_fselector_title("Save a Dataset");
	filename = fl_show_fselector("Dataset to Save","","*.lda","");
	if(filename == NULL)
	{
		say_status("Ready");
		reenable_input();
		return;
	}
	fptr = fopen(filename,"w");
	if(!fptr)
	{
		fl_show_alert("Could not open requested file for writing","","",1);
		reenable_input();
		say_status("Ready");
		return;
	}
	strcpy(data_matrix_description,remove_blanks(data_matrix_description));
	if(strlen(data_matrix_description) < 1) strcpy(data_matrix_description,"_");
	for(count = 0; count < numb_variables; count++)
	{
		strcpy(data_matrix[count].description,remove_blanks(data_matrix[count].description));
		if(strlen(data_matrix[count].description) < 1) strcpy(data_matrix[count].description,"_");
		strcpy(data_matrix[count].name,remove_blanks(data_matrix[count].name));
	}
	fprintf(fptr,"%-s\n",data_matrix_description);
	fprintf(fptr,"%-f\n",missing_value);
	for(count = 0; count < numb_variables; count++)
	{
		fprintf(fptr,"%s\n",data_matrix[count].name);
		fprintf(fptr,"%s\n",data_matrix[count].description);
		fprintf(fptr,"%i\n",data_matrix[count].obs);
		
		for(countt = 0; countt < data_matrix[count].obs; countt++)
		{
			fprintf(fptr,"%-f\n",*(fvector[count] + countt));
		}
	}
	strcpy(data_matrix_description,add_blanks(data_matrix_description));
	for(count = 0; count < numb_variables; count++)
	{
		strcpy(data_matrix[count].description,add_blanks(data_matrix[count].description));
		strcpy(data_matrix[count].name,add_blanks(data_matrix[count].name));
	}
	oktoquit=TRUE;
	sprintf(string_one,"Saved %d variable(s) in %s",numb_variables,filename);
	simple_line_output("save",string_one);
	reenable_input();
	fclose(fptr);
	say_status("Ready");
}

void append_a_variable(FILE *lfptr)
{
	char scratch[XLDLASMAX_INPUT];
	int count, exists_number;
	int extra;
	if(numb_variables == MAX_VARS)
	{
		fl_show_alert("Dataset Contains Too Many Variables",
				"It was probably built with a bigger version of xldlas",
				"Please adjust MAX_VARS in xldlas.h and recompile xldlas",1);
		drain_input(lfptr);
		return;
	}
	fscanf(lfptr,"%s\n",scratch);
	exists_number = -1;
	for(count = 0; count < numb_variables; count++)
	{
		if(strcmp(data_matrix[count].name, add_blanks(scratch)) == 0) exists_number = count;
	}
	if(exists_number == -1)
	{
		strncpy(data_matrix[numb_variables].name, scratch,NAME_LENGTH);
		fvector[numb_variables] = (float *) malloc (MAX_OBS * sizeof(float));
		if (!fvector[numb_variables])
		{
			fl_show_alert("Cannot Load all data in Dataset",
					"Running low on Memory",
					"",1);
			drain_input(lfptr);
			return;
		}
		fscanf(lfptr,"%s\n",scratch);
		strncpy(data_matrix[numb_variables].description, scratch,DESC_LENGTH);
		fscanf(lfptr,"%i\n",&data_matrix[numb_variables].obs);
		data_matrix[numb_variables].fmem_start=fvector[numb_variables];
		for(count = 0; count < data_matrix[numb_variables].obs; count ++)
		{
			fscanf(lfptr,"%f\n", &*(fvector[numb_variables] + count));
		}
		numb_variables++;
	}
	else
	{
		fscanf(lfptr,"%s\n",scratch);
		fscanf(lfptr,"%i\n",&extra);
		for(count = 0; count < extra; count++)
		{
			fscanf(lfptr,"%f\n",&*(fvector[exists_number] + data_matrix[exists_number].obs  + count));
		}
		data_matrix[exists_number].obs = data_matrix[exists_number].obs + extra;
	}
}


void append_dataset(FL_OBJECT *ob, long user_data)
{
	FILE *lfptr;
	char trash[XLDLASMAX_INPUT];
	float ignore;
	char string_one[XLDLASMAX_INPUT];
	int count, temp;
	const char *filename;
	inhibit_input();
	say_status("Loading Dataset");
	fl_use_fselector(2);
	fl_set_fselector_title("Append a Dataset");
	filename = fl_show_fselector("Dataset to Append","","*.lda","");
	if(filename == NULL)
	{
		say_status("Ready");
		reenable_input();
		return;
	}
	lfptr = fopen(filename,"r");
	if(!lfptr)
	{
		fl_show_alert("Could not open requested file for reading","","",1);
		reenable_input();
		say_status("Ready");
		return;
	}
	fscanf(lfptr,"%s\n",trash);
	fscanf(lfptr,"%f\n",&ignore);
	temp = 0;
	while(!feof(lfptr))
	{
		temp++;
		append_a_variable(lfptr);
	}
	fclose(lfptr);
	for(count = 0; count <= numb_variables; count++)
	{
		strcpy(data_matrix[count].description,add_blanks(data_matrix[count].description));
		strcpy(data_matrix[count].name,add_blanks(data_matrix[count].name));
	}
	data_exists = TRUE;
	oktoquit = FALSE;
	sprintf(string_one,"Read/Appended %d variable(s) into memory from %s", temp, filename);
	simple_line_output("apnd",string_one);
	reenable_input();
	say_status("Ready");
}


int count_columns(FILE *filepointer)
{
	int escape, numb_columns;
	char one_character;
	char last_character;
	escape = FALSE;
	last_character = ' ';
	numb_columns = 0;
	while(!feof(filepointer) && escape == FALSE)
	{
		fscanf(filepointer,"%c", &one_character);
		if(one_character == '\n')
		{
			if(last_character != ' ' && last_character != '\t') numb_columns++;
			escape = TRUE;
		}
		if(one_character == ' ' && last_character != ' ' && last_character != '\t')
		{
			numb_columns++;
		}
		if(one_character == '\t' && last_character != ' ' && last_character != '\t')
		{
			numb_columns++;
		}
		last_character = one_character;
	}
	return(numb_columns);
}		

int import_a_row(FILE *filepointer)
{
	char slow_build[XLDLASMAX_INPUT];
	char varname[NAME_LENGTH];
	char stupid[2];
	char one_character;
	char last_character;
	int place;
	int escape, count;
	count = 0;
	sprintf(varname,"unknown%i", count);
	while(check_variable_name(varname) == FALSE)
	{
		count++;
		sprintf(varname,"unknown%i", count);
	}
	strcpy(data_matrix[numb_variables].name,varname);	
	fvector[numb_variables] = (float *) malloc (MAX_OBS * sizeof(float));
	escape = FALSE;
	place = 0;
	strcpy(slow_build,"");
	last_character = ' ';
	while(escape == FALSE)
	{
		fscanf(filepointer,"%c",&one_character);
		if(one_character == '\n' || feof(filepointer))
		{
			if(last_character != ' ' && last_character != '\t')
			{
				*(fvector[numb_variables] + place) = atof(slow_build);
				place++;
			}
			escape = TRUE;
		}
		else
		{
			if(one_character != ' ' && one_character != '\t')
			{
				stupid[0] = one_character;
				stupid[1] = '\0';
				strcat(slow_build,stupid);
			}
			else
			{
				if(last_character != ' ' && last_character != '\t')
				{
					*(fvector[numb_variables] + place) = atof(slow_build);
					place++;
					if(place >= MAX_OBS)
					{
						fl_show_alert("There are More Observations than Xldlas can Hold!", "Try changing the #define MAX_OBS in xldlas.h","I'll add extra observations in this row to a new variable",TRUE);
						escape = TRUE;
					}
					strcpy(slow_build,"");
				}
			}
			last_character = one_character;
		}
	}
	if(place == 0)
	{
		free(fvector[numb_variables]);
		strcpy(data_matrix[numb_variables].name,"");
		return(0);
	}
	data_matrix[numb_variables].obs = place;
	numb_variables++;
	return(1);
}


void import_dataset(FL_OBJECT *ob, long user_data)
{
	FILE *lfptr;
	int choice, numb_imports, count, countt, nameok, place, name_start;
	char string_one[XLDLASMAX_INPUT];
	char varname[MAX_VARS][NAME_LENGTH];
	const char *filename;
	inhibit_input();
	say_status("Importing Dataset");
	fl_use_fselector(3);
	fl_set_fselector_title("Import a Dataset");
	filename = fl_show_fselector("Dataset to Append","","*","");
	if(filename == NULL)
	{
		say_status("Ready");
		reenable_input();
		return;
	}
	lfptr = fopen(filename,"r");
	if(!lfptr)
	{
		fl_show_alert("Could not open requested file for reading","","",1);
		reenable_input();
		say_status("Ready");
		return;
	}

	choice = fl_show_choice("ASCII data is in which format?","","",
               		3,"Columns","Rows","Cancel");
	if(choice == 3)
	{
		reenable_input();
		say_status("Ready");
		fclose(lfptr);
		return;
	}
	if(choice == 2) import_type = TYPE_ROWS;
	if(choice == 1) import_type = TYPE_COLS;

	if(import_type == TYPE_COLS)
	{
		numb_imports = count_columns(lfptr);
		fclose(lfptr);
		if(numb_imports == 0)
		{
			fl_show_alert("I can't find anything in the specified file","","",1);
			fclose(lfptr);
			reenable_input();
			say_status("Ready");
			return;
		}
		if(numb_imports > MAX_VARS - numb_variables)
		{
			fl_show_alert("No room to add all these variables","Try changing #define MAX_VARS in xldlas.h","I'm not importing anything",1);
			fclose(lfptr);
			reenable_input();
			say_status("Ready");
			return;
		}
		nameok = FALSE;
		name_start = 0;
		for(countt = 0; countt < MAX_VARS; countt++)
		{
			sprintf(varname[0],"unknown%i", countt);
			if(check_variable_name(varname[0]) == TRUE)
			{
				name_start = countt;
				countt = MAX_VARS + 1;
				nameok = TRUE;
			}
		}
		if(nameok == FALSE)
		{
			fl_show_alert("Ran out of Space for Names!","","",1);
			fclose(lfptr);
			reenable_input();
			say_status("Ready");
			return;
		}
		for(count = 0; count < numb_imports; count++)
		{
			sprintf(varname[count],"unknown%i", count + name_start);
		}
		for(count = 0; count < numb_imports; count++)
		{
			strncpy(data_matrix[numb_variables + count].name, varname[count],NAME_LENGTH);
			fvector[numb_variables + count] = (float *) malloc (MAX_OBS * sizeof(float));
			if (!fvector[numb_variables])
			{
				fl_show_alert("Cannot Load all data in Dataset",
						"Running low on Memory",
						"",1);
				drain_input(lfptr);
				fclose(lfptr);
				return;
			}
		}
		lfptr = fopen(filename,"r");
		place = 0;
		while(!feof(lfptr))
		{
			nameok = FALSE;
			for(count = 0; count < numb_imports; count++)
			{
				if(!feof(lfptr))
				{
					if(place >= MAX_OBS)
					{
						fl_show_alert("There are More Observations than Xldlas can Hold!", "Try changing the #define MAX_OBS in xldlas.h","I'm truncating this data.",TRUE);
						nameok = FALSE;
						drain_input(lfptr);
					}
					else
					{
						fscanf(lfptr,"%f",&*(fvector[numb_variables + count] + place));
						if(*(fvector[numb_variables + count] + place) != 0) nameok = TRUE;
					}
				}
			}
			if(nameok == TRUE && (!feof(lfptr))) place++;
		}
		for(count = 0; count < numb_imports; count++)
		{
			data_matrix[numb_variables + count].obs = place;
		}
		numb_variables = numb_variables + numb_imports;
	}
	else
	{
		lfptr = fopen(filename,"r");
		numb_imports = 0;
		while(!feof(lfptr))
		{
			if(numb_imports + numb_variables > MAX_VARS - 1)
			{
				fclose(lfptr);
				fl_show_alert("Too Many Rows of Variables",
						"",
						"",1);

				data_exists = TRUE;
				oktoquit = FALSE;
				sprintf(string_one,"Imported %d variable(s) from %s", numb_imports, filename);
				simple_line_output("imprt",string_one);
				reenable_input();
				say_status("Ready");
				return;
				
			}
			else
			{
				numb_imports = numb_imports + import_a_row(lfptr);
			}
		}
		
	}
	fclose(lfptr);
	data_exists = TRUE;
	oktoquit = FALSE;
	sprintf(string_one,"Imported %d variable(s) from %s", numb_imports, filename);
	simple_line_output("imprt",string_one);
	reenable_input();
	say_status("Ready");
}





void file_routines(FL_OBJECT *menu, long user_data)
{
	int choice;
	choice = fl_get_menu(menu);
	if(choice == 1) about_xldlas(menu, user_data);
	if(choice == 2) load_dataset(menu, user_data);
	if(choice == 3) save_dataset(menu, user_data);
	if(choice == 4) append_dataset(menu, user_data);
	if(choice == 5) import_dataset(menu, user_data);
	if(choice == 6) quit_xldlas(menu, user_data);
}




