/*      -------------------------------------------------------------------
	xldlas -- A Stastics Package

	Copyright (C) 1996 Thor Sigvaldason

	Handles all graphing routines, both the oncsreen ones using
	xforms routines and the calling of gnuplot for exporting graphs 
	to all the file formats supported by gnuplot
	
        -------------------------------------------------------------------*/

#include "xldlas.h"

/* need all of this stuff on VMS to set up the pipe as this program expects*/
#ifdef __VMS
#define GNUPLOT_CMD "GNUPLOT"
#include <unixio.h>
#include <file.h>
#include <starlet.h>
#include <syidef.h>
#include <dvidef.h>
#include <libdef.h>
#include <lib$routines.h>
#include <ssdef.h>
#include <stsdef.h>
#include <descrip.h>

/* have to put the subprocess PID and CHAN to mailbox somewhere safe
   so that they can be removed later */

#endif

extern void simple_line_output(char which_routine[XLDLASMAX_INPUT], char the_output[XLDLASMAX_INPUT]);
extern void begin_column_output(char the_output[XLDLASMAX_INPUT], int justify);
extern void add_column_output(char the_output[XLDLASMAX_INPUT], int justify);
extern void end_column_output();
extern void seperator_output(int how_many);
extern void begin_table_output(int how_many, char title[XLDLASMAX_INPUT]);
extern void end_table_output(int how_many);

extern void inhibit_input();
extern void reenable_input();

extern void say_status(char the_status[XLDLASMAX_INPUT]);

extern void graph_output(char save_name[XLDLASMAX_INPUT], int the_xvar, int the_variables[], int how_many);

extern void sort_working_vector();

#ifdef __VMS
extern void remove_gnuplot(int );
extern int  create_mbx(unsigned short int *, struct dsc$descriptor_s *);
#endif


void set_graph_range_counters(FL_OBJECT *obj, long arg)
{
	int i,j;
	float biggest, smallest;
	int to_check[MAX_VARS], numb_check;
	int start_check, stop_check;
	if(arg == 1 && fl_get_button(graph_xrange_auto_button) == 0)
	{
		start_check = fl_get_counter_value(graph_from_counter);
		stop_check = fl_get_counter_value(graph_to_counter);
		if(stop_check < start_check)
		{
			stop_check = 1;
			start_check = 1;
		}
		numb_check = -1;
		for(i=0 ; i< numb_variables; i++)
		{
			if(fl_isselected_browser_line(xvar_browser, i+1))
			{
				numb_check = i;
				i = numb_variables;
			}
		}
		if(numb_check > -1)
		{
			biggest = *(fvector[numb_check] + start_check - 1);
			smallest = *(fvector[numb_check] + start_check - 1);
			for(i = start_check; i < stop_check; i ++)
			{
				if(i < data_matrix[numb_check].obs)
				{
					if(smallest == missing_value || smallest > *(fvector[numb_check] + i))
					{
						smallest = *(fvector[numb_check] + i);
					}
					if(biggest == missing_value || biggest < *(fvector[numb_check] + i))
					{
						biggest = *(fvector[numb_check] + i);
					}
				}
			}
			fl_set_counter_value(graph_xrange_from_counter, smallest);
			fl_set_counter_step(graph_xrange_from_counter, 0.1, (biggest - smallest) / 20);
			fl_set_counter_value(graph_xrange_to_counter, biggest);
			fl_set_counter_step(graph_xrange_to_counter, 0.1, (biggest - smallest) / 20);
		}	
	}
	if(arg == 2 && fl_get_button(graph_yrange_auto_button) == 0)
	{
		start_check = fl_get_counter_value(graph_from_counter);
		stop_check = fl_get_counter_value(graph_to_counter);
		if(stop_check < start_check)
		{
			stop_check = 1;
			start_check = 1;
		}
		numb_check = 0;
		for(i=0 ; i< numb_variables; i++)
		{
			if(fl_isselected_browser_line(yvar_browser, i+1))
			{
				to_check[numb_check] = i;
				numb_check++;
			}
		}
		if(numb_check > 0)
		{
			biggest = *(fvector[to_check[0]] + start_check - 1);
			smallest = *(fvector[to_check[0]] + start_check - 1);
			for(i = start_check; i < stop_check; i ++)
			{
				for(j = 0; j < numb_check; j++)
				{
					if(i < data_matrix[to_check[j]].obs)
					{
						if(smallest == missing_value || smallest > *(fvector[to_check[j]] + i))
						{
							smallest = *(fvector[to_check[j]] + i);
						}
						if(biggest == missing_value || biggest < *(fvector[to_check[j]] + i))
						{
							biggest = *(fvector[to_check[j]] + i);
						}
					}
				}
			}
			fl_set_counter_value(graph_yrange_from_counter, smallest);
			fl_set_counter_step(graph_yrange_from_counter, 0.1, (biggest - smallest) / 20);
			fl_set_counter_value(graph_yrange_to_counter, biggest);
			fl_set_counter_step(graph_yrange_to_counter, 0.1, (biggest - smallest) / 20);
		}	
	}
}

void do_system_call(char the_command[XLDLASMAX_INPUT])
{
	int error;
	error = system(the_command);
	if(error == 127 || error == -1)
	{
		fl_show_alert("The following system call failed:", the_command, "", TRUE);
	}
}



void graph_set_bar(FL_OBJECT *obj, long arg)
{
	numb_bars = rint(fl_get_counter_value(graph_bar_counter));
}

void graph_toggles(FL_OBJECT *obj, long arg)
{
	if(arg == 1)
	{
		if(graph_key_on == TRUE)
			graph_key_on = FALSE;
		else
			graph_key_on = TRUE; 
	}
	if(arg == 2)
	{
		if(graph_overwrite_on == TRUE)
			graph_overwrite_on = FALSE;
		else
			graph_overwrite_on = TRUE; 
	}
	if(arg == 3)
	{
		if(graph_normal_on == TRUE)
			graph_normal_on = FALSE;
		else
			graph_normal_on = TRUE; 
	}
	fl_set_button(graph_normal_button, 0);
	fl_set_button(graph_normal_button, 0);
	fl_set_button(graph_overwrite_button, 0);
	if(graph_key_on == TRUE)
	{
		fl_set_button(graph_key_button, 1);
	}
	if(graph_normal_on == TRUE)
	{
		fl_set_button(graph_normal_button, 1);
	}
	if(graph_overwrite_on == TRUE)
	{
		fl_set_button(graph_overwrite_button, 1);
	}
}

void done_attributes(FL_OBJECT *obj, void *arg)
{
	line_style[temp_varnumber] = rint(fl_get_counter_value(att_line_counter));
	point_style[temp_varnumber] = rint(fl_get_counter_value(att_point_counter));
	fl_hide_form(att_window);
}

int click_done_attributes(FL_FORM *form, long arg)
{
	line_style[temp_varnumber] = rint(fl_get_counter_value(att_line_counter));
	point_style[temp_varnumber] = rint(fl_get_counter_value(att_point_counter));
	return(FL_OK);
}

/* Yes folks, we do things a little differently on OpenVMS...*/
#ifdef __VMS
void remove_gnuplot(int fd)
{
        close(fd);                 /* close the C io */
        sys$delprc(gnuplot_pid);   /* kill off the subprocess */
        sys$delmbx(gnuplot_chan);  /* stomp the mailbox */
}

int launch_gnuplot()
{
    static char mbxname[64];
    unsigned short int chan;
    unsigned long int pid, flags=1;  /* nowait - gnu c doesn't allow &1 */
    unsigned long int status;
    int fd ;
    int oflags = O_RDWR;
    struct dsc$descriptor_s namdsc = {sizeof mbxname, DSC$K_DTYPE_T,
                                      DSC$K_CLASS_S, mbxname},
                            cmddsc = {0, DSC$K_DTYPE_T,
                                      DSC$K_CLASS_S, 0};


    cmddsc.dsc$w_length=strlen(GNUPLOT_CMD);
    cmddsc.dsc$a_pointer=GNUPLOT_CMD;

    /* create mailbox, note, it comes up with the name */
    
    status = create_mbx(&chan, &namdsc);
    if(status != 1){
	fl_show_alert("I Can't Start GNUPlot (create_mbx() failed)","","",TRUE);
	return(-1);
    }

    /* open a FILE * into it */

    fd=open(mbxname, oflags, 0);

    if(fd == -1){
        sys$delmbx(chan);
	fl_show_alert("I Can't Start GNUPlot (fopen() failed)","","",TRUE);
	return(-1);
    }

    status = lib$spawn(&cmddsc, /* command string */
                       &namdsc, /* input file */
                       0,       /* sys$output */
                       &flags,  /* flags */
                       0,       /* process name */
                       &pid,    /* process ID */
                       0, /* completion status address */
                       0, 0, 0,0,0,0);

    if(status != 1){
        close(fd);
        sys$delmbx(chan);
	fl_show_alert("I Can't Start GNUPlot (lib$spawn() failed)","","",TRUE);
	return(-1);
    }

    /* gnuplot is running, put it into X11 mode */

    write(fd,"set term X11\n",13);

    /* store the PID and CHAN for future use in getting rid of the
       subprocess/mailbox */
    gnuplot_pid = pid;
    gnuplot_chan = chan;

    return(fd);
}

/* almost, but not quite, verbatim from perl VMS.C - this one does NOT 
   check status returned by system routines */

int create_mbx(unsigned short int *chan, struct dsc$descriptor_s *namdsc)
{
  static unsigned long int mbxbufsiz;
  unsigned long status;
  long int syiitm = SYI$_MAXBUF, dviitm = DVI$_DEVNAM;

  if (!mbxbufsiz) {
    /*
     * Get the SYSGEN parameter MAXBUF, and the smaller of it and the
     * preprocessor consant BUFSIZ from stdio.h as the size of the
     * 'pipe' mailbox.
     */
    lib$getsyi(&syiitm, &mbxbufsiz, 0, 0, 0, 0);
    if (mbxbufsiz > BUFSIZ) mbxbufsiz = BUFSIZ;
  }
  status = sys$crembx(0,chan,mbxbufsiz,mbxbufsiz,0,0,0);
  if(status != 1)return(0);

  status = lib$getdvi(&dviitm, chan, NULL, NULL, namdsc, &namdsc->dsc$w_length);
  namdsc->dsc$a_pointer[namdsc->dsc$w_length] = '\0';
  if(status != 1)return(0);

  return(1);

}  /* end of create_mbx() */

#else  /* __VMS */
int launch_gnuplot()
{
	int gnu_pipes[2];
	pipe(gnu_pipes);
	gnuplot_pid = fork();
	switch(gnuplot_pid)
	{
		case -1:	fl_show_alert("I Can't Start GNUPlot (fork() failed)","","",TRUE);
				return(-1);

		case 0:		close(0);
				dup(gnu_pipes[0]);
				close(gnu_pipes[0]);
				close(gnu_pipes[1]);
				execlp("gnuplot", "gnuplot", "-geometry", gnuplot_geometry, "-title", "'Gnuplot via xldlas'", (char *)0);
				fl_show_alert("Hmmmmm  Can't seem to run GNUPlot","","",TRUE);
				break;

		case 1:		close(gnu_pipes[0]);
	}
	return(gnu_pipes[1]);	
}
#endif /* __VMS */



void handle_graph_choices(FL_OBJECT *obj, long arg)
{
	fl_set_button(choose_hist_button,0);
	fl_set_button(choose_plot_button,0);
	if(arg == 1)
	{
		fl_set_button(choose_hist_button,1);
	}
	else
	{
		fl_set_button(choose_plot_button,1);
	}
}

void handle_graph_browsers(FL_OBJECT *obj, long arg)
{
	int which;
	char string_one[XLDLASMAX_INPUT];
	which = fl_get_browser(yvar_browser);
	if(which > 0)
	{
		temp_varnumber = which - 1;
		fl_set_counter_value(att_line_counter, line_style[which - 1]);
		fl_set_counter_bounds(att_line_counter, 0, 7);
		fl_set_counter_step(att_line_counter, 1, 1);
		fl_set_counter_value(att_point_counter, point_style[which - 1]);
		fl_set_counter_bounds(att_point_counter, 0, 7);
		fl_set_counter_step(att_point_counter, 1, 1);
		sprintf(string_one,"Plot Attributes of %s", data_matrix[which-1].name);
		fl_show_form(att_window,FL_PLACE_FREE,FL_TRANSIENT,string_one);
	}
}

void do_logo_plot()
{
	char string_one[XLDLASMAX_INPUT];
	tmpnam(gnuplot_datafile_name);
	gnuplot_datafile = fopen(gnuplot_datafile_name, "w");
	fprintf(gnuplot_datafile, "1 1\n");
	fprintf(gnuplot_datafile, "3 4\n");
	fprintf(gnuplot_datafile, "\n");
	fprintf(gnuplot_datafile, "1 4\n");
	fprintf(gnuplot_datafile, "3 1\n");
	fprintf(gnuplot_datafile, "\n");
	fprintf(gnuplot_datafile, "4 1\n");
	fprintf(gnuplot_datafile, "4 7\n");
	fprintf(gnuplot_datafile, "\n");
	fprintf(gnuplot_datafile, "5 1\n");
	fprintf(gnuplot_datafile, "5 4\n");
	fprintf(gnuplot_datafile, "7 4\n");
	fprintf(gnuplot_datafile, "7 1\n");
	fprintf(gnuplot_datafile, "5 1\n");
	fprintf(gnuplot_datafile, "\n");
	fprintf(gnuplot_datafile, "7 1\n");
	fprintf(gnuplot_datafile, "7 7\n");
	fprintf(gnuplot_datafile, "\n");
	fprintf(gnuplot_datafile, "8 1\n");
	fprintf(gnuplot_datafile, "8 7\n");
	fprintf(gnuplot_datafile, "\n");
	
	fprintf(gnuplot_datafile, "11 2.5\n");
	fprintf(gnuplot_datafile, "11 1\n");
	fprintf(gnuplot_datafile, "9 1\n");
	fprintf(gnuplot_datafile, "9 2.5\n");
	fprintf(gnuplot_datafile, "11 2.5\n");
	fprintf(gnuplot_datafile, "11 4\n");
	fprintf(gnuplot_datafile, "9 4\n");
	fprintf(gnuplot_datafile, "\n");
	fprintf(gnuplot_datafile, "12 1\n");
	fprintf(gnuplot_datafile, "14 1\n");
	fprintf(gnuplot_datafile, "14 2.5\n");
	fprintf(gnuplot_datafile, "12 2.5\n");
	fprintf(gnuplot_datafile, "12 4\n");
	fprintf(gnuplot_datafile, "14 4\n");
#ifdef __VMS  /* the datafile must be closed before it is loaded
                 because it was opened for write access only and will
                 be locked.*/
	fclose(gnuplot_datafile);
#endif
	write(gnuplot_pipe, "set xrange[0:15]\n", strlen("set xrange[0:15]\n"));
	write(gnuplot_pipe, "set yrange[0:8]\n", strlen("set yrange[0:9]\n"));
	write(gnuplot_pipe, "set nokey\n", strlen("set nokey\n")); 
	sprintf(string_one, "plot '%s' using 1:2 w l 1\n",gnuplot_datafile_name);
	write(gnuplot_pipe, string_one, strlen(string_one));
#ifndef __VMS
	fclose(gnuplot_datafile);
#endif
}

int save_graph(char graph_name[XLDLASMAX_INPUT])
{
	char string_one[XLDLASMAX_INPUT];
	FILE *file_pointer;
	file_pointer = fopen(graph_name,"r");
	if(file_pointer && graph_overwrite_on == FALSE)
	{
		fl_show_alert("Save File already Exists","","",TRUE);
		return(FALSE);
	}
	fclose(file_pointer);
	strcpy(graph_output_format, fl_get_input(graph_format_input));
	strcpy(graph_output_options, fl_get_input(graph_options_input));
	sprintf(string_one,"set term %s %s\n", graph_output_format, graph_output_options);
	write(gnuplot_pipe, string_one, strlen(string_one));
	sprintf(string_one,"set output '%s'\n", graph_name);
	write(gnuplot_pipe, string_one, strlen(string_one));
	write(gnuplot_pipe, "replot\n", strlen("replot\n"));
	write(gnuplot_pipe,"set term x11\n", strlen("set term x11\n"));
	write(gnuplot_pipe,"set output\n",strlen("set output\n"));
	write(gnuplot_pipe, "replot\n", strlen("replot\n"));
	return(TRUE);
}

void graph_variables()
{
	int i,j,k;
	int toplot[MAX_VARS];
	int numbplot;
	int xvar;
	char string_one[XLDLASMAX_INPUT];
	char string_two[XLDLASMAX_INPUT];
	numbplot = 0;
	xvar = -1;
	for(i=0; i < numb_variables;i++)
	{
		if(fl_isselected_browser_line(xvar_browser, i+1))
		{
			xvar = i;
			i = numb_variables;
		}
	}
	if(xvar == -1)
	{
		fl_show_alert("No X variable selected","","",TRUE);
		return;
	}
	for(i=0; i < numb_variables;i++)
	{
		if(fl_isselected_browser_line(yvar_browser, i+1))
		{
			toplot[numbplot] = i;
			numbplot++;
		}
	}
	if(numbplot == 0)
	{
		fl_show_alert("No Y variable(s) selected","","",TRUE);
		return;
	}
	all_start = rint(fl_get_counter_value(graph_from_counter));
	all_stop = rint(fl_get_counter_value(graph_to_counter));
	if(all_stop < all_start)
	{
		fl_show_alert("From Value Greater than to Value!",
			      "",
			      "",
			      TRUE);
		return;
	}
	gnuplot_datafile = fopen(gnuplot_datafile_name,"w");
	write(gnuplot_pipe,"set autoscale\n", strlen("set autoscale\n"));
	if(fl_get_button(graph_xrange_auto_button) != 1)
	{
		if(fl_get_counter_value(graph_xrange_from_counter) == fl_get_counter_value(graph_xrange_to_counter))
		{
			fl_set_counter_value(graph_xrange_to_counter, fl_get_counter_value(graph_xrange_from_counter) + 0.1);
		}	
		sprintf(string_one,"set xrange[%f:%f]\n", fl_get_counter_value(graph_xrange_from_counter), fl_get_counter_value(graph_xrange_to_counter));
		write(gnuplot_pipe, string_one, strlen(string_one));
	}
	if(fl_get_button(graph_yrange_auto_button) != 1)
	{
		if(fl_get_counter_value(graph_yrange_from_counter) == fl_get_counter_value(graph_yrange_to_counter))
		{
			fl_set_counter_value(graph_yrange_to_counter, fl_get_counter_value(graph_yrange_from_counter) + 0.1);
		}	
		sprintf(string_one,"set yrange[%f:%f]\n", fl_get_counter_value(graph_yrange_from_counter), fl_get_counter_value(graph_yrange_to_counter));
		write(gnuplot_pipe, string_one, strlen(string_one));
	}
	if(graph_key_on == TRUE) write(gnuplot_pipe, "set key\n", strlen("set key\n"));
	else write(gnuplot_pipe, "set nokey\n", strlen("set nokey\n"));
	for(j = all_start - 1; j < all_stop; j++)
	{
		if(*(fvector[xvar] + j) == missing_value || data_matrix[xvar].obs <= j)
		{
			fprintf(gnuplot_datafile,"\n");
		}
		else
		{
			fprintf(gnuplot_datafile, "%f\t",*(fvector[xvar] + j));
			for(k =0; k < numbplot; k++)
			{
				if(data_matrix[toplot[k]].obs <= j)
				{
					fprintf(gnuplot_datafile,";\t");
				}
				else
				{
					if(*(fvector[toplot[k]] + j) == missing_value)
						fprintf(gnuplot_datafile,";\t");
					else
						fprintf(gnuplot_datafile,"%f\t", *(fvector[toplot[k]] + j));
				}
			}
			fprintf(gnuplot_datafile,"\n");
		}
	}
	fclose(gnuplot_datafile);
	strcpy(string_one,fl_get_input(graph_title_input));
	if(strlen(string_one) > 0)
	{
		string_two[0]=\0;
		strcat(string_two,"set title \"");
		strcat(string_two,string_one);
		strcat(string_two,"\"\n");
		write(gnuplot_pipe,string_two,strlen(string_two)); 
	}
	else
	{
		write(gnuplot_pipe,"set title\n", strlen("set title\n"));
	}
	strcpy(string_one,fl_get_input(graph_xlabel_input));
	if(strlen(string_one) > 0)
	{
		string_two[0]=\0;
		strcat(string_two,"set xlabel \"");
		strcat(string_two,string_one);
		strcat(string_two,"\"\n");
		write(gnuplot_pipe,string_two,strlen(string_two)); 
	}
	else
	{
		write(gnuplot_pipe,"set xlabel\n", strlen("set xlabel\n"));
	}
	strcpy(string_one,fl_get_input(graph_ylabel_input));
	if(strlen(string_one) > 0)
	{
		string_two[0]=\0;
		strcat(string_two,"set ylabel \"");
		strcat(string_two,string_one);
		strcat(string_two,"\"\n");
		write(gnuplot_pipe,string_two,strlen(string_two)); 
	}
	else
	{
		write(gnuplot_pipe,"set ylabel\n", strlen("set ylabel\n"));
	}

/*The VMS "pipe" puts line feeds on the end as it has to go through a 
  fairly narrow mailbox, so modify the commands to have line continuations on 
  them*/
#ifdef __VMS
	write(gnuplot_pipe,"plot \\", strlen("plot \\"));
#else
	write(gnuplot_pipe,"plot ", strlen("plot "));
#endif
	for(i = 0; i < numbplot; i++)
	{
		if(line_style[toplot[i]] == 0)
		{
#ifdef __VMS
			sprintf(string_one,"'%s.;' using 1:%d title '%s' w points %d ",
                         gnuplot_datafile_name,i + 2, data_matrix[toplot[i]].name,point_style[toplot[i]]);
			if(i != numbplot - 1) strcat(string_one,", \\");
#else
			sprintf(string_one,"'%s' using 1:%d title '%s' w points %d ", gnuplot_datafile_name,i + 2, data_matrix[toplot[i]].name,point_style[toplot[i]]);
			if(i != numbplot - 1) strcat(string_one,", ");
#endif
			write(gnuplot_pipe, string_one, strlen(string_one));
		}
		else
		{
#ifdef __VMS
			sprintf(string_one,"'%s.;' using 1:%d title '%s' w linespoints %d %d ",
                          gnuplot_datafile_name,i + 2, data_matrix[toplot[i]].name, line_style[toplot[i]],point_style[toplot[i]]);
			if(i != numbplot - 1) strcat(string_one,", \\");
#else
			sprintf(string_one,"'%s' using 1:%d title '%s' w linespoints %d %d ", gnuplot_datafile_name,i + 2, data_matrix[toplot[i]].name, line_style[toplot[i]],point_style[toplot[i]]);
			if(i != numbplot - 1) strcat(string_one,", ");
#endif
			write(gnuplot_pipe, string_one, strlen(string_one));
		}  
	}
	write(gnuplot_pipe,"\n", strlen("\n"));
	strcpy(string_one,fl_get_input(graph_save_input));
	i = FALSE;
	if(strlen(string_one) > 0)
	{
		i = save_graph(string_one);
		if(i == TRUE) graph_output(string_one, xvar, toplot, numbplot);
	}
	if(i == FALSE) graph_output("", xvar, toplot, numbplot);
}

void hist_variables()
{
	int i,j;
	int xvar;
	float xcord, ycord, running, runningt, average, variance;
	char string_one[XLDLASMAX_INPUT];
	xvar = -1;
	for(i=0; i < numb_variables;i++)
	{
		if(fl_isselected_browser_line(xvar_browser, i+1))
		{
			xvar = i;
			i = numb_variables;
		}
	}
	if(xvar == -1)
	{
		fl_show_alert("No X variable selected","","",TRUE);
		return;
	}
	all_start = rint(fl_get_counter_value(graph_from_counter));
	all_stop = rint(fl_get_counter_value(graph_to_counter));
	if(all_stop > data_matrix[xvar].obs) all_stop = data_matrix[xvar].obs;
	
	if(all_stop < all_start)
	{
		fl_show_alert("From Value Greater than to Value!",
			      "",
			      "",
			      TRUE);
		return;
	}
	gnuplot_datafile = fopen(gnuplot_datafile_name,"w");
	for(i = 0; i < all_stop - (all_start - 1); i++)
	{
		working[i] = *(fvector[xvar] + i + (all_start -1));
	}	
	worksize = all_stop - (all_start - 1);
	sort_working_vector();
	xcord=working[1];
	for(i = 1; i <= numb_bars; i++)
	{
		ycord = 0;
		for(j = 1;j <= worksize; j++)
		{
			if(working[j] >= working[1] + (i - 1) * ((working[worksize - 1] - working[1]) / numb_bars) && working[j] < working[1] + (i * ((working[worksize - 1] - working[1]) / numb_bars))) ycord++;
		}
		if(i == numb_bars) ycord++;
		fprintf(gnuplot_datafile,"%f  0.00\n", xcord);
		fprintf(gnuplot_datafile,"%f   %f\n", xcord, ycord / worksize);
		xcord = xcord + ((working[worksize - 1] - working[1]) / numb_bars);
		fprintf(gnuplot_datafile,"%f   %f\n", xcord, ycord / worksize);
		fprintf(gnuplot_datafile,"%f  0.00\n",xcord);
	}
	fclose(gnuplot_datafile);
	write(gnuplot_pipe,"set autoscale\n", strlen("set autoscale\n"));
	if(fl_get_button(graph_xrange_auto_button) != 1)
	{
		if(fl_get_counter_value(graph_xrange_from_counter) == fl_get_counter_value(graph_xrange_to_counter))
		{
			fl_set_counter_value(graph_xrange_to_counter, fl_get_counter_value(graph_xrange_from_counter) + 0.1);
		}	
		sprintf(string_one,"set xrange[%f:%f]\n", fl_get_counter_value(graph_xrange_from_counter), fl_get_counter_value(graph_xrange_to_counter));
		write(gnuplot_pipe, string_one, strlen(string_one));
	}
	if(graph_key_on == TRUE) write(gnuplot_pipe, "set key\n", strlen("set key\n"));
	else write(gnuplot_pipe, "set nokey\n", strlen("set nokey\n"));
	strcpy(string_one,fl_get_input(graph_title_input));
	if(strlen(string_one) > 0)
	{
		write(gnuplot_pipe,"set title \"", strlen("set title \""));
		strcat(string_one,"\"\n");
		write(gnuplot_pipe,string_one,strlen(string_one)); 
	}
	else
	{
		write(gnuplot_pipe,"set title\n", strlen("set title\n"));
	}
	strcpy(string_one,fl_get_input(graph_xlabel_input));
	if(strlen(string_one) > 0)
	{
		write(gnuplot_pipe,"set xlabel \"", strlen("set xlabel \""));
		strcat(string_one,"\"\n");
		write(gnuplot_pipe,string_one,strlen(string_one)); 
	}
	else
	{
		write(gnuplot_pipe,"set xlabel\n", strlen("set xlabel\n"));
	}
	strcpy(string_one,fl_get_input(graph_ylabel_input));
	if(strlen(string_one) > 0)
	{
		write(gnuplot_pipe,"set ylabel \"", strlen("set ylabel \""));
		strcat(string_one,"\"\n");
		write(gnuplot_pipe,string_one,strlen(string_one)); 
	}
	else
	{
		write(gnuplot_pipe,"set ylabel\n", strlen("set ylabel\n"));
	}
	if(graph_normal_on == FALSE)
	{
		sprintf(string_one,"plot '%s' title '%s' with lines\n", gnuplot_datafile_name, data_matrix[xvar].name);
		write(gnuplot_pipe,string_one, strlen(string_one));
	}
	else 
	{
		running = 0;
		runningt = 0;
		for(i = 0; i < worksize; i++)
		{
			running = running + working[i];
			runningt = runningt + (working[i] * working[i]);
		}
		average = running / worksize;
		variance = (runningt / worksize) - (average * average);
		
		sprintf(string_one,"plot '%s' title '%s' with lines, ((exp(-0.5 * (((x - %f) / sqrt(%f)) * ((x - %f) / sqrt(%f))))) / (sqrt(%f) * sqrt(6.2831853))) * %f title 'normal' \n",
		   gnuplot_datafile_name, data_matrix[xvar].name, average,variance,average,variance,variance, (working[worksize - 1] - working[1]) / numb_bars);
		write(gnuplot_pipe,string_one,strlen(string_one));
	}
	strcpy(string_one,fl_get_input(graph_save_input));
	i = FALSE;
	if(strlen(string_one) > 0)
	{
		i = save_graph(string_one);
		if(i == TRUE) graph_output(string_one, xvar, 0, 0);
	}
	if(i == FALSE) graph_output("", xvar, 0, 0);
}

void plot_variables(FL_OBJECT *obj, long arg)
{
	if(fl_get_button(choose_plot_button) == TRUE)
	{
		graph_variables();
	}
	else
	{
		hist_variables();
	}
}




int click_done_graph_variables(FL_FORM *form, long arg)
{
	char string_one[XLDLASMAX_INPUT];
	write(gnuplot_pipe, "exit\n",strlen("exit\n"));
#ifdef __VMS
        remove_gnuplot(gnuplot_pipe);
	sprintf(string_one," delete %s..*", gnuplot_datafile_name);
#else
	close(gnuplot_pipe);
	wait(0);
	sprintf(string_one," rm %s &", gnuplot_datafile_name);
#endif
	do_system_call(string_one);
	say_status("Ready");
	reenable_input();
	return(FL_OK);
}

void done_graph_variables(FL_FORM *form, void *arg)
{
	char string_one[XLDLASMAX_INPUT];
	write(gnuplot_pipe, "exit\n",strlen("exit\n"));
#ifdef __VMS
        remove_gnuplot(gnuplot_pipe);
	sprintf(string_one," delete %s.;*", gnuplot_datafile_name);
#else
	close(gnuplot_pipe);
	wait(0);
	sprintf(string_one," rm %s &", gnuplot_datafile_name);
#endif
	do_system_call(string_one);
	fl_hide_form(graph_window);
	say_status("Ready");
	reenable_input();
}




void start_graph_variables(FL_OBJECT *obj, long arg)
{
	int i;
	int largest;
	inhibit_input();
	say_status("Graphing Variables");
	fl_clear_browser(xvar_browser);
	fl_clear_browser(yvar_browser);
	largest = 0;
	for(i=0; i < numb_variables; i++)
	{
		if(data_matrix[i].obs > largest) largest = data_matrix[i].obs;
		fl_addto_browser(xvar_browser,data_matrix[i].name);
		fl_addto_browser(yvar_browser,data_matrix[i].name);
	}

	fl_set_counter_value(graph_from_counter, 1);
	fl_set_counter_bounds(graph_from_counter, 1, largest);
	fl_set_counter_step(graph_from_counter, 1, 10);

	fl_set_counter_value(graph_to_counter, largest);
	fl_set_counter_bounds(graph_to_counter, 1, largest);
	fl_set_counter_step(graph_to_counter, 1, 10);
	
	fl_set_counter_value(graph_bar_counter, numb_bars);
	fl_set_counter_bounds(graph_bar_counter, 1, XLDLAS_NUMB_BARS_MAX);
	fl_set_counter_step(graph_bar_counter, 1, 1);

	fl_set_button(choose_hist_button,0);
	fl_set_button(choose_plot_button,0);

	if(arg == 1)
	{
		fl_set_button(choose_hist_button,1);
	}
	else
	{
		fl_set_button(choose_plot_button,1);
	}

	fl_set_button(graph_key_button, 0);
	if(graph_key_on == TRUE)
	{
		fl_set_button(graph_key_button, 1);
	}
	
	fl_set_button(graph_overwrite_button, 0);
	if(graph_overwrite_on == TRUE)
	{
		fl_set_button(graph_overwrite_button, 1);
	}

	fl_set_button(graph_normal_button, 0);
	if(graph_normal_on == TRUE)
	{
		fl_set_button(graph_normal_button, 1);
	}
	
	
	fl_set_input(graph_format_input,graph_output_format);
	fl_set_input(graph_options_input,graph_output_options);
	for(i = 0; i < numb_variables; i++)
	{
		point_style[i] = 0;
		line_style[i] = (i % 6) + 1 ;
	}	
	
	fl_show_form(graph_window,FL_PLACE_FREE,FL_FULLBORDER,"Graph");
	gnuplot_pipe = launch_gnuplot();
	do_logo_plot();
}



void graph_routines(FL_OBJECT *menu, long user_data)
{
	int choice;
	choice = fl_get_menu(menu);
	if(choice == 1) start_graph_variables(menu, 1);
	if(choice == 2) start_graph_variables(menu, 2);
}


